import VBMicrolensing
import pandas as pd
import numpy as np
import math
from scipy.optimize import minimize,least_squares
import RTModel
import shutil
import os
import matplotlib.pyplot as plt
import time
import logging
import sys
import multiprocessing as mp

from jasmine.files_organizer import ra_and_dec_conversions as radec
from pyLIMA import event
from pyLIMA import telescopes
from pyLIMA.models import PSPL_model,FSPL_model
from pyLIMA.fits import DE_fit,TRF_fit

def calculate_magnifications_pspl(pars,data_list,data_mag,ndatasets,VBMInstance):
    """Calculate LC magnitudes in each passband provided for PSPL model"""
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    pars_log = [np.log(pars[0]), np.log(pars[1]), pars[2]]
    for i in range(ndatasets):
        data = data_list[i]
        magnifications, y1, y2 = VBMInstance.PSPLLightCurve(pars_log,data[:,-1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        magnifications, y1, y2 = VBMInstance.PSPLLightCurve(pars_log,data_mag)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    return magnitude_list,source_flux_list,blend_flux_list

def evaluate_model_plx(psplpars, fsblpars, a1_list, data_list, VBMInstance, pspl_chi2):
    """ Calculate chi2 for one model in the grid
        fsblpars = [s,q,alpha,tstar]
        psplpars = follows VBMicrolensing format
        data_list list with a dataframe for each band in RTModel data format
        ndatasets = number of datasets
    """
    #blend_flux_list = []
    #source_flux_list = []
    ndatasets = len(data_list)
    pars = [np.log(fsblpars[0]), np.log(fsblpars[1]), psplpars[0], fsblpars[2], np.log(fsblpars[3] / psplpars[1]),
            math.log(psplpars[1]), psplpars[2],psplpars[3],psplpars[4]]
    magnitude_list,source_flux_list,blend_flux_list = calculate_magnifications(pars,a1_list,data_list,ndatasets,VBMInstance,parallax=True)
    magdata_list = []
    magerr_list = []
    for i in range(ndatasets):
        magdata_list.append(data_list[i][:,0])
        magerr_list.append(data_list[i][:,1])
    chi2_list,chi2_sum = get_chi2(magnitude_list, magdata_list, magerr_list, ndatasets)
    return_list = np.concatenate((pars, source_flux_list,blend_flux_list, chi2_list,[chi2_sum, chi2_sum-pspl_chi2]))
    return return_list

def create_sp_icgs_grid(event_path,pspl_pars,q_grid, alpha_frac, s_frac, n_sep):
    s_list = []
    q_list = []
    alpha_list = []

    # Set pspl parameters from fit.
    u0 = pspl_pars[0]
    tE = pspl_pars[1]
    t0 = pspl_pars[2]
    # read RTModel peak positions
    peaks =  read_RTModel_initcond(event_path)
    d_t0_peaks = np.abs(peaks - t0)
    ind_t0_peak = np.argmin(d_t0_peaks)

    q = 0.01
    for index in range(len(peaks)):
        if index == ind_t0_peak: continue
        dt = (peaks[index] - t0) / tE;
        xc0 = math.sqrt(u0 ** 2 + dt ** 2)
        alpha0 = math.atan2(u0, -dt)
        xc = xc0;
        s0 = 0.5 * (math.sqrt(4 + xc * xc) + xc);
        while (xc < 4 * math.sqrt(q) / (s0 * s0)):
            q *= 0.1
            print(q)
        q_indices = np.where(q_grid <= q)[0]

        for q_index in range(q_grid.shape[0]):  # q_indices:
            q = q_grid[q_index]
            # +
            xc = xc0 + 4 * math.sqrt(q) / (s0 * s0);
            s_center = 0.5 * (math.sqrt(4 + xc * xc) + xc);
            s_grid = np.logspace(math.log(s_center * (1 - s_frac)), math.log(s_center * (1 + s_frac)), base=np.e,
                                 num=n_sep)
            alpha_center = alpha0;
            alpha_grid = np.arange(alpha_center * (1 - alpha_frac), alpha_center * (1 + alpha_frac), step=alpha_frac)
            s_sublist, q_sublist, alpha_sublist = np.meshgrid(s_grid, q, alpha_grid)
            s_sublist = s_sublist.flatten()
            q_sublist = q_sublist.flatten()
            alpha_sublist = alpha_sublist.flatten()
            s_list.append(s_sublist)
            q_list.append(q_sublist)
            alpha_list.append(alpha_sublist)
            print(s_center, q, alpha_center)
            # -
            xc = xc0 - 4 * math.sqrt(q) / (s0 * s0)
            s_center = 0.5 * (math.sqrt(4 + xc * xc) + xc)
            s_grid = np.logspace(math.log(s_center * (1 - s_frac)), math.log(s_center * (1 + s_frac)), base=np.e,
                                 num=n_sep)
            alpha_grid = np.arange(alpha_center * (1 - alpha_frac), alpha_center * (1 + alpha_frac), step=alpha_frac)
            s_sublist, q_sublist, alpha_sublist = np.meshgrid(s_grid, q, alpha_grid)
            s_sublist = s_sublist.flatten()
            q_sublist = q_sublist.flatten()
            alpha_sublist = alpha_sublist.flatten()
            s_list.append(s_sublist)
            q_list.append(q_sublist)
            alpha_list.append(alpha_sublist)
            print(s_center, q, alpha_center)

        xc = xc0;
        s0 = 0.5 * (math.sqrt(4 + xc * xc) - xc);
        q = 0.01;
        while (xc < 3 * math.sqrt(3 * q) * s0 * s0 * s0):
            q *= 0.1;
        q_indices = np.where(q_grid <= q)[0]
        for q_index in q_indices:
            # -sqrt(3q)
            # +asin
            q = q_grid[q_index]
            xc = xc0 - 3 * math.sqrt(3 * q) * s0 * s0 * s0;
            s_center = 0.5 * (math.sqrt(4 + xc * xc) - xc);
            print(f's {s_center}')
            try:
                alpha_center = alpha0 + math.pi + math.asin(
                    np.abs(2 * math.sqrt(q * (1 - s_center * s_center)) / s_center) / xc);

                s_grid = np.logspace(math.log(s_center * (1 - s_frac)), math.log(s_center * (1 + s_frac)), base=np.e,
                                     num=n_sep)
                alpha_grid = np.arange(alpha_center * (1 - alpha_frac), alpha_center * (1 + alpha_frac),
                                       step=alpha_frac)
                s_sublist, q_sublist, alpha_sublist = np.meshgrid(s_grid, q, alpha_grid)
                s_sublist = s_sublist.flatten()
                q_sublist = q_sublist.flatten()
                alpha_sublist = alpha_sublist.flatten()
                s_list.append(s_sublist)
                q_list.append(q_sublist)
                alpha_list.append(alpha_sublist)
                print(s_center, q, alpha_center)
                # - asin
                alpha_center = alpha0 + math.pi - math.asin(
                    np.abs(2 * math.sqrt(q * (1 - s_center * s_center)) / s_center) / xc);
                s_grid = np.logspace(math.log(s_center * (1 - s_frac)), math.log(s_center * (1 + s_frac)), base=np.e,
                                     num=n_sep)
                alpha_grid = np.arange(alpha_center * (1 - alpha_frac), alpha_center * (1 + alpha_frac),
                                       step=alpha_frac)
                s_sublist, q_sublist, alpha_sublist = np.meshgrid(s_grid, q, alpha_grid)
                s_sublist = s_sublist.flatten()
                q_sublist = q_sublist.flatten()
                alpha_sublist = alpha_sublist.flatten()
                s_list.append(s_sublist)
                q_list.append(q_sublist)
                alpha_list.append(alpha_sublist)
                print(s_center, q, alpha_center)
            except ValueError:
                print('Bad sqrt')
                print(f'Problematic s = {s_center}')
            # +sqrt(3q)
            # +asin
            xc = xc0 + 3 * math.sqrt(3 * q) * s0 * s0 * s0;
            s_center = 0.5 * (math.sqrt(4 + xc * xc) - xc);
            try:
                alpha_center = alpha0 + math.pi + math.asin(
                    np.abs(2 * math.sqrt(q * (1 - s_center * s_center)) / s_center) / xc);
                s_grid = np.logspace(math.log(s_center * (1 - s_frac)), math.log(s_center * (1 + s_frac)), base=np.e,
                                     num=n_sep)
                alpha_grid = np.arange(alpha_center * (1 - alpha_frac), alpha_center * (1 + alpha_frac),
                                       step=alpha_frac)
                s_sublist, q_sublist, alpha_sublist = np.meshgrid(s_grid, q, alpha_grid)
                s_sublist = s_sublist.flatten()
                q_sublist = q_sublist.flatten()
                alpha_sublist = alpha_sublist.flatten()
                s_list.append(s_sublist)
                q_list.append(q_sublist)
                alpha_list.append(alpha_sublist)
                print(s_center, q, alpha_center)
                # -asin
                alpha_center = alpha0 + math.pi - math.asin(
                    np.abs(2 * math.sqrt(q * (1 - s_center * s_center)) / s_center) / xc);
                s_grid = np.logspace(math.log(s_center * (1 - s_frac)), math.log(s_center * (1 + s_frac)), base=np.e,
                                     num=n_sep)
                alpha_grid = np.arange(alpha_center * (1 - alpha_frac), alpha_center * (1 + alpha_frac),
                                       step=alpha_frac)
                s_sublist, q_sublist, alpha_sublist = np.meshgrid(s_grid, q, alpha_grid)
                s_sublist = s_sublist.flatten()
                q_sublist = q_sublist.flatten()
                alpha_sublist = alpha_sublist.flatten()
                s_list.append(s_sublist)
                q_list.append(q_sublist)
                alpha_list.append(alpha_sublist)
                print(s_center, q, alpha_center)
            except ValueError:
                print('Bad sqrt')
                print(f'Problematic s = {s_center}')

    s_list = np.concatenate(s_list)
    q_list = np.concatenate(q_list)
    alpha_list = np.concatenate(alpha_list)
    return s_list, q_list, alpha_list



def fspl_fit_pyLIMA(event_path,dataset_list,a1_list):
    data_list = []
    for i in range(len(dataset_list)):
        data_list.append(np.loadtxt(f'{event_path}/Data/{dataset_list[i]}'))
        data_list[i] = data_list[i][:, [2, 0, 1]]
        data_list[i][:,0]+=2450000 # add +2_450_000 for pyLIMA
    print('Using pyLIMA DE')

    file_path = f'{event_path}/Data/event.coordinates'
    event_coordinates = pd.read_csv(file_path, delim_whitespace=True, header=None, names=['RA_HMS', 'Dec_DMS'])
    # Convert RA and Dec to decimal degrees
    ra_deg = radec.ra_hms_to_deg(event_coordinates['RA_HMS'].iloc[0])
    dec_deg = radec.dec_dms_to_deg(event_coordinates['Dec_DMS'].iloc[0])

    your_event = event.Event(ra=ra_deg, dec=dec_deg)
    your_event.name = f'PSPL Fit'
    telescope_1 = telescopes.Telescope(name='RomanW146',
                                       camera_filter='W146',
                                       lightcurve=data_list[0],
                                       lightcurve_names=['time', 'mag', 'err_mag'],
                                       lightcurve_units=['JD', 'mag', 'mag'])
    telescope_2 = telescopes.Telescope(name='RomanZ087',
                                       camera_filter='Z087',
                                       lightcurve=data_list[1],
                                       lightcurve_names=['time', 'mag', 'err_mag'],
                                       lightcurve_units=['JD', 'mag', 'mag'])
    telescope_3 = telescopes.Telescope(name='RomanK213',
                                       camera_filter='K213',
                                       lightcurve=data_list[2],
                                       lightcurve_names=['time', 'mag', 'err_mag'],
                                       lightcurve_units=['JD', 'mag', 'mag'])

    telescope_1.ld_a1 = a1_list[0]
    telescope_2.ld_a1 = a1_list[1]
    telescope_3.ld_a1 = a1_list[2]

    your_event.telescopes.append(telescope_1)
    your_event.telescopes.append(telescope_2)
    your_event.telescopes.append(telescope_3)
    your_event.find_survey('RomanW146')
    your_event.check_event()
    print(telescope_1.bad_data)
    fspl = FSPL_model.FSPLmodel(your_event, parallax=['None', 0.0])

    diffev_fit = DE_fit.DEfit(fspl, loss_function='chi2')
    diffev_fit.fit_parameters['u0'][1] = [0., 5.]
    diffev_fit.fit()
    # Do gradient fit to fine tune solution
    gradient_fit = TRF_fit.TRFfit(fspl, loss_function='chi2')
    gradient_fit.fit_parameters['u0'][1] = [0, 5.]  # PyLima has a short limit for u0

    gradient_fit.model_parameters_guess = diffev_fit.fit_results['best_model'][0:3]
    gradient_fit.fit()


    results = gradient_fit.fit_results['best_model']
    chi2 = gradient_fit.fit_results['chi2']
    return results,chi2

def pspl_fit(event_path,dataset_list,p0=None,init_ind = 0,method='lm', t0_grid = True):
    """ Find best fitting PSPL Model"""
    data_list = []
    for i in range(len(dataset_list)):
        data_list.append(np.loadtxt(f'{event_path}/Data/{dataset_list[i]}'))
    VBMInstance = VBMicrolensing.VBMicrolensing()
    VBMInstance.RelTol = 1e-03
    VBMInstance.Tol=1e-03
    if p0 is None:
        print('Setting default initial conditions')
        t0_init = data_list[init_ind][np.argmin(data_list[init_ind][:,0]),-1]
        p0 = [0.1,10,t0_init]
    else:
        print('Using provided initial conditions')
        t0_init = p0[-1]
    #magnitude_list = []
    #source_flux_list = []
    #blend_flux_list = []
    if method =='lm':
        res_arr = []
        print('Using Levenberg-Marquardt')
        print('Small grid in t0')
        p0[-1] = t0_init - 1
        res = least_squares(fun=calc_pspl_residuals, x0=p0, args=[[data_list, len(data_list), VBMInstance]],
                            method='lm', ftol=1e-10, xtol=1e-10, gtol=1e-10, max_nfev=50000)
        res_arr.append(res)
        print('----------')
        p0[-1] = t0_init + 0
        res = least_squares(fun=calc_pspl_residuals, x0=p0, args=[[data_list,len(data_list),VBMInstance]],method='lm',ftol=1e-10, xtol=1e-10, gtol=1e-10, max_nfev=50000)
        res_arr.append(res)

        print('----------')
        p0[-1] = t0_init + 1
        res = least_squares(fun=calc_pspl_residuals, x0=p0, args=[[data_list, len(data_list), VBMInstance]],
                            method='lm', ftol=1e-10, xtol=1e-10, gtol=1e-10, max_nfev=50000)
        res_arr.append(res)
        print('----------')
        chi2 = []
        for i in range(3):
            chi2.append(res_arr[i].cost*2)
        min_ind = np.argmin(chi2)
        res = res_arr[min_ind]
    else:
        print('Using some scipy.minimize')
        res = minimize(calc_pspl_chi2, p0,[data_list,len(data_list),VBMInstance],method=method)
    print('Done Fitting PSPL!')
    return res


def calc_pspl_chi2(pars,args):
    """ Objective function to optimize for pspl_fit() """
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    data_list = args[0]
    ndatasets = args[1]
    VBMInstance = args[2]
    #logger = args[3]
    pars_log = [np.log(pars[0]),np.log(pars[1]),pars[2]]
    #logger.info(pars)
    for i in range(ndatasets):
        data = data_list[i]
        magnifications,y1,y2 = VBMInstance.PSPLLightCurve(pars_log,data[:,-1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    magdata_list = []
    magerr_list = []
    for i in range(ndatasets):
        magdata_list.append(data_list[i][:,0])
        magerr_list.append(data_list[i][:,1])
    chi2_list, chi2 = get_chi2(magnitude_list, magdata_list, magerr_list, ndatasets)
    print(chi2)
    return chi2

def calc_pspl_residuals(pars,args):
    """ Objective function to optimize for pspl_fit() """
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    data_list = args[0]
    ndatasets = args[1]
    VBMInstance = args[2]
    #logger = args[3]
    pars_log = [np.log(pars[0]),np.log(pars[1]),pars[2]]
    #print(pars)
    for i in range(ndatasets):
        data = data_list[i]
        magnifications,y1,y2 = VBMInstance.PSPLLightCurve(pars_log,data[:,-1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    magdata_list = []
    magerr_list = []
    for i in range(ndatasets):
        magdata_list.append(data_list[i][:,0])
        magerr_list.append(data_list[i][:,1])
    residuals = get_residuals(magnitude_list, magdata_list, magerr_list, ndatasets)
    chi2 = np.sum(np.power(residuals,2))
    print(f'chi2: {chi2}')
    return residuals
def read_RTModel_initcond(event_path):
    ''' Under Construction'''
    with open(f'{event_path}/InitCond/InitCondPS.txt', 'r') as f:
        line1 = f.readline().split(' ')
        npeaks = int(line1[0])
        peaks = []
        for ipeak in range(npeaks):
            peaks.append(float(f.readline().split(' ')[0]))
    return np.array(peaks)

def run_event_rtm_grid(event_path,dataset_list,grid_q,tstar,d_alpha, n_alpha, alpha_frac, s_frac, n_sep, a1_list,pspl_thresh,processors,satellitedir,method='lm'):
    """ Wrapper Function to go from pspl_fit to final RTModel runs.
        Runs it from a user provided meshgrid instead of constructing the grid from input arrays"""
    # First do the PSPL fit
    pspl_results = pspl_fit(event_path=event_path,dataset_list=dataset_list,method=method)
    if method == 'lm':
        pspl_chi2 = pspl_results.cost*2
    else: pspl_chi2 = pspl_results.chi2
    pspl_pars = pspl_results.x
    #save pspl fit to a txt file

    rtm = RTModel.RTModel()
    rtm.set_processors(nprocessors=processors)
    rtm.set_event(event_path)
    rtm.archive_run()
    shutil.rmtree(f'{event_path}/run-0001') # have to remove old stuff or it affects the InitConds for everything.
    modeltypes = ['PS','LS','LX','LO']
    rtm.set_satellite_dir(satellitedir=satellitedir)
    peak_threshold = 5
    rtm.set_processors(nprocessors=processors)
    rtm.set_event(event_path)
    rtm.set_satellite_dir(satellitedir=satellitedir)
    rtm.config_Reader(otherseasons=0, binning=1000000)
    rtm.config_InitCond(usesatellite=1, peakthreshold=peak_threshold,modelcategories=modeltypes)
    rtm.Reader()
    rtm.InitCond()

    time0 = time.time()
    s_list, q_list, alpha_list = create_sp_icgs_grid(event_path, pspl_pars,grid_q, alpha_frac, s_frac, n_sep)
    grid_fit_results = grid_fit_pass_meshgrid(event_path=event_path, dataset_list=dataset_list, pspl_pars=pspl_pars,
                                s=s_list,q=q_list,alpha=alpha_list,tstar=tstar,a1_list=a1_list,pspl_chi2=pspl_chi2)
    time1 = time.time()
    print(f'ICGS time: {time1-time0}')
    time0 = time.time()
    names =  ['log(s)','log(q)','u0','alpha','log(rho)','log(tE)','t0','fs0','fb0','fs1','fb1','fs2','fb2','chi20','chi21','chi22','chi2sum','delta_pspl_chi2']
    grid_result_df = pd.DataFrame(grid_fit_results,columns=names)
    filtered_df,init_conds = filter_by_q(grid_result=grid_result_df,pspl_thresh=pspl_thresh)
    #Now run these in RTModel
    # Have RTModel prints go to log not stdout
    #Write some outputs after clearing the directory
    with open(f'{event_path}/Data/pspl_pars.txt','w') as f:
        f.write(f'{pspl_pars[0]},{pspl_pars[1]},{pspl_pars[2]},{pspl_chi2}')
    np.savetxt(fname=f'{event_path}/Data/ICGS_initconds.txt', X=init_conds)  # save init conds to a text file
    np.savetxt(f'{event_path}/Data/grid_fit.txt', grid_fit_results)
    #Do FSPL fit for comparison
    print('Launching PS Fits')
    rtm.launch_fits('PS')
    rtm.ModelSelector('PS')
    print('Launching LS Fits')
    num_init_cond = init_conds.shape[0]

    for n in range(num_init_cond):
        init_cond = list(init_conds[n,:])
        #launch each fit from the init conds
        rtm.LevMar(f'LSfit{n:03}',parameters = init_cond)
    rtm.ModelSelector('LS')
    print('Launching LX and LO fits')
    rtm.launch_fits('LX')
    rtm.ModelSelector('LX')
    rtm.launch_fits('LO')
    rtm.ModelSelector('LO')
    rtm.Finalizer()
    print('Done')
    time1 = time.time()
    print(f'RTModel time: {time1 - time0}')
    return None

def plot_pspl(pars,dataset,event_path):
    data = np.loadtxt(f'{event_path}/Data/{dataset}')
    VBMInstance = VBMicrolensing.VBMicrolensing()
    VBMInstance.RelTol = 1e-03
    VBMInstance.Tol = 1e-03
    tmag = np.linspace(data[0,-1],data[-1,-1],80000)
    magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_pspl(pars,[data],tmag,1,VBMInstance)
    magnitude = magnitude_list[0]
    fig, ax = plt.subplots(dpi=100, layout='tight')
    ax.plot(tmag, magnitude, zorder=10, label='PSPL Fit', color='black')
    #magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_pspl([0.6291636,50.20121621,pars[-1]], [data],tmag, 1, VBMInstance)
    #magnitude = magnitude_list[0]
    #ax.plot(tmag, magnitude, zorder=10, label='PSPL True', color='red')
    ax.errorbar(data[:, -1], y=data[:, 0], yerr=data[:, 1], marker='.', markersize=0.75,
                linestyle=' ', label='W146')
    ax.set_xlim(pars[-1]-1*pars[1],pars[-1]+1*pars[1])
    ax.yaxis.set_inverted(True)
    ax.legend()
    plt.savefig(f'{event_path}/pspl_plot.png')
    plt.show()
    return None




def calculate_magnifications_plots(pars,a1_list,data_list,data_mag,ndatasets,VBMInstance,parallax=False):
    """Calculate LC magnitudes in each passband provided for 2L1S"""
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    for i in range(ndatasets):
        VBMInstance.a1 = a1_list[i]
        data = data_list[i]
        if parallax == False:
            magnifications, y1, y2 = VBMInstance.BinaryLightCurve(pars,data[:,-1])
        else:
            magnifications, y1, y2, sorb = VBMInstance.BinaryLightCurveOrbital(pars, data[:, -1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        #Fix so minimize_linear_parameters uses flux_err not mag_err
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        magnifications, y1, y2 = VBMInstance.BinaryLightCurve(pars, data_mag)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    return magnitude_list,source_flux_list,blend_flux_list

def plot_2L1S(pars,dataset,event_path,xrange,yrange,evname):
    data = np.loadtxt(f'{event_path}/Data/{dataset}')
    VBMInstance = VBMicrolensing.VBMicrolensing()
    tmag = np.linspace(pars[-1]-5*pars[5],pars[-1]+5*pars[5],80000)
    a1 = 0.33
    magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_plots(pars,[a1],[data],tmag,1,VBMInstance)
    magnitude = magnitude_list[0]
    fig, ax = plt.subplots(dpi=100, layout='tight')
    ax.plot(tmag, magnitude, zorder=10, label='2L1S Fit', color='black')
    #magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_pspl([0.6291636,50.20121621,pars[-1]], [data],tmag, 1, VBMInstance)
    #magnitude = magnitude_list[0]
    #ax.plot(tmag, magnitude, zorder=10, label='PSPL True', color='red')
    ax.errorbar(data[:, -1], y=data[:, 0], yerr=data[:, 1], marker='.', markersize=0.75,
                linestyle=' ', label='W146')
    if yrange == None:
        ind_0 = np.where(tmag >= xrange[0])
        ind_1 = np.where(tmag <= xrange[1])

        ind_data_0 = np.where(data[:,-1] >= xrange[0])
        ind_data_1 = np.where(data[:, -1] <= xrange[1])
        ind_plot = np.intersect1d(ind_0,ind_1)
        ind_data = np.intersect1d(ind_data_0,ind_data_1)

        ymin = np.min([np.min(magnitude[ind_plot]) - 0.1,np.min(data[ind_data,0]) - 0.1])
        ymax = np.max([np.max(magnitude[ind_plot]) + 0.1,np.max(data[ind_data,0]) + 0.1])
        yrange=[ymin,ymax]
    ax.set_xlim(xrange[0],xrange[1])
    ax.set_ylim(yrange[0], yrange[1])
    ax.yaxis.set_inverted(True)
    ax.set_ylabel('W146')
    ax.set_xlabel("HJD'")
    ax.legend()
    plt.savefig(f'{event_path}/{evname}')
    plt.show()
    return None

def plot_2L1S_parallax(pars,dataset,event_path,xrange,yrange,evname,satellitedir):
    data = np.loadtxt(f'{event_path}/Data/{dataset}')
    VBMInstance = VBMicrolensing.VBMicrolensing()
    VBMInstance.RelTol = 1e-04
    VBMInstance.Tol = 1e-05
    coordinatefile = f'{event_path}/Data/event.coordinates'
    VBMInstance.SetObjectCoordinates(coordinatefile, satellitedir)
    VBMInstance.satellite=1
    tmin = pars[6]-5*pars[5]
    tmax = pars[6]+5*pars[5]
    if tmin < data[0,-1]:
        tmin = data[0,-1]
    if tmax > data[-1,-1]:
        tmax = data[-1,-1]
    tmag = np.linspace(tmin,tmax,80000)
    a1 = 0.33
    magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_plots(pars,[a1],[data],tmag,1,VBMInstance,parallax=True)
    magnitude = magnitude_list[0]
    fig, ax = plt.subplots(dpi=100, layout='tight')
    ax.plot(tmag, magnitude, zorder=10, label='2L1S Fit', color='black')
    #magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_pspl([0.6291636,50.20121621,pars[-1]], [data],tmag, 1, VBMInstance)
    #magnitude = magnitude_list[0]
    #ax.plot(tmag, magnitude, zorder=10, label='PSPL True', color='red')
    ax.errorbar(data[:, -1], y=data[:, 0], yerr=data[:, 1], marker='.', markersize=0.75,
                linestyle=' ', label='W146')
    # Set some good ylim based on the xlim if yrange==None.
    if yrange == None:
        ind_0 = np.where(tmag >= xrange[0])
        ind_1 = np.where(tmag <= xrange[1])

        ind_data_0 = np.where(data[:,-1] >= xrange[0])
        ind_data_1 = np.where(data[:, -1] <= xrange[1])
        ind_plot = np.intersect1d(ind_0,ind_1)
        ind_data = np.intersect1d(ind_data_0,ind_data_1)

        ymin = np.min([np.min(magnitude[ind_plot]) - 0.1,np.min(data[ind_data,0]) - 0.1])
        ymax = np.max([np.max(magnitude[ind_plot]) + 0.1,np.max(data[ind_data,0]) + 0.1])
        yrange=[ymin,ymax]
    ax.set_xlim(xrange[0],xrange[1])
    ax.set_ylim(yrange[0], yrange[1])
    ax.yaxis.set_inverted(True)
    ax.set_ylabel('W146')
    ax.set_xlabel("HJD'")
    ax.legend()
    plt.savefig(f'{event_path}/{evname}')
    plt.show()
    return None


import pandas as pd
import numpy as np
import re

def hms_to_seconds(hms):
    h, m, s = map(int, hms.split(':'))
    return h * 3600 + m * 60 + s



def main(list_of_events_path_, log_names_path_, event_run_path_, logfile_base_path_):
    list_of_events_ = pd.read_table(list_of_events_path_, header=None)
    list_of_logfiles = pd.read_table(log_names_path_, header=None)
    stats = []
    for run_index in range(len(list_of_events_)):
        event_name = list_of_events_.iloc[run_index, 0]
        print(f"Event: {event_name}")
        grid_fit_path = f'{event_run_path_}/{event_name}/Data/grid_fit.txt'
        grid_df = pd.read_csv(grid_fit_path, delim_whitespace=True, names=['grid_index', 'chi2', 'calculation_time'])
        min_time = grid_df['calculation_time'].min()
        print(f"min_time: {min_time}")
        max_time = grid_df['calculation_time'].max()
        print(f"max_time: {max_time}")
        median_time = grid_df['calculation_time'].median()
        print(f"median_time: {median_time}")
        mean_time = grid_df['calculation_time'].mean()
        logfile_name = list_of_logfiles.iloc[run_index, 0]
        logfile_path = f'{logfile_base_path_}/{logfile_name}'
        with open(logfile_path, 'r') as f:
            for line in f:
                icgs_match = re.search(r'ICGS time:\s*([\d\.]+)', line)
                rtmodel_match = re.search(r'RTModel time:\s*([\d\.]+)', line)
                walltime_used_match = re.search(r'Walltime Used                   :\s*([\d:]+)', line)
                if icgs_match:
                    print(icgs_match)
                    icgs_times = float(icgs_match.group(1)) * 10
                    icgs_in_hours = icgs_times / 3600
                if rtmodel_match:
                    print(rtmodel_match)
                    rtmodel_times = float(rtmodel_match.group(1)) * 10
                    rtmodel_in_hours = rtmodel_times / 3600
                if walltime_used_match:
                    print(walltime_used_match)
                    walltime_used = walltime_used_match.group(1)
                    walltime_used_in_seconds = hms_to_seconds(walltime_used) * 10
                    walltime_used_in_hours = walltime_used_in_seconds / 3600
        stats.append({
            'event_name': event_name,
            'VBM_min_time_seconds': min_time,
            'VBM_max_time_seconds': max_time,
            'VBM_median_time_seconds': median_time,
            'VBM_mean_time_seconds': mean_time,
            'ICGS_time_total_hours': icgs_in_hours,
            'RTModel_time_total_hours': rtmodel_in_hours,
            'CPU_hours': f"{walltime_used_in_hours:.3f}",
            # 'ICGS_time_seconds': icgs_times,
            # 'RTModel_time': rtmodel_times,
            # 'Walltime': walltime_used_in_seconds,
        })
        print(grid_df.head())

    time_statistic = pd.DataFrame(stats)
    print(time_statistic)

    time_statistic.to_csv('time_statistics_slow_runs.csv', index=False)

if __name__ == "__main__":

    root_path = f'/gpfsm/dnb34/sishitan/orbital_task'
    event_run_path = f'{root_path}/RTModel_runs/154_failures_v24_v31/second_run'
    logfile_base_path = root_path
    list_of_events_path = f'{root_path}/RTModel_runs/154_failures_v24_v31/list_rerun_26.txt'
    log_names_path = f'{root_path}/RTModel_runs/154_failures_v24_v31/second_run/slurm_files_names.txt'
    main(list_of_events_path, log_names_path, event_run_path, logfile_base_path)


import os
import re
import pandas as pd


def main(directory_with_events_, master_file_, output_csv_):
    master_df = pd.read_csv(master_file_)
    master_df['event_name_'] = (
        master_df['lcname'].str.replace('OMPLDG_croin_cassan/OMPLDG_croin_cassan_', 'event_', regex=False).str.replace(
            '.det.lc', '', regex=False))

    # Prepare CSV columns
    header = ["event_name_",
              "found_a_planetary_solution",
              "found_a_2L1S_solution",
              "complete_classification",
              "models"]
    rows = []

    # Finding full classification descriptions for each event
    for event_name in os.listdir(directory_with_events_):
        event_path = os.path.join(directory_with_events_, event_name)

        # Skip non-directories and hidden files
        if not os.path.isdir(event_path) or event_name.startswith('.'):
            continue

        # DEBUG LINE
        # # try to read Nature.txt file and if not found, skip the event
        # if not os.path.isfile(os.path.join(event_path, "Nature.txt")):
        #     print(f"Nature.txt not found for event: {event_name}, skipping.")
        #     continue

        # Read Nature.txt file
        nature_file = os.path.join(event_path, "Nature.txt")
        with open(nature_file, "r") as f:
            lines = f.readlines()
            content = ''.join(lines)

        # Extract the line starting with 'Successful:'
        complete_classification = ""
        match = re.search(r"Successful:\s*(.*)", content)
        if match:
            complete_classification = match.group(1).strip()
        found_a_planet = "Planetary lens" in complete_classification # Save if planetary solution found

        # Find the start of the 'chisquare   model' section
        try:
            start = next(i for i, line in enumerate(lines) if "chisquare" in line)
        except StopIteration:
            start = None

        models = []
        if start is not None:
            for line in lines[start + 1:]:
                parts = line.split()
                if len(parts) != 2 or not parts[1].endswith(".txt"):
                    break
                models.append(parts[1].replace(".txt", ""))

        # Check if any model starts with 'L'
        found_a_2L1S_solution = any(m.startswith("L") for m in models)
        rows.append([event_name, found_a_planet, found_a_2L1S_solution, complete_classification, models])

    # Create DataFrame
    df = pd.DataFrame(rows, columns=header)
    df = df.sort_values(by='event_name_')
    # Save to CSV
    df.to_csv(output_csv_, index=False)
    print(f"Saved DataFrame to: {output_csv_}")


if __name__ == '__main__':
    # EXAMPLE OF HOW TO USE THIS SCRIPT
    # general_path = '/Users/stela/Documents/Scripts/orbital_task'
    general_path = '/gpfsm/dnb34/sishitan/orbital_task'

    # Directory containing all event folders
    directory_with_events = f"{general_path}/RTModel_runs/154_failures_v24_v31/second_run"
    # Master CSV file
    master_file = f'{general_path}/data/gulls_orbital_motion_extracted/OMPLDG_croin_cassan.sample.csv'
    # Output CSV
    output_csv = f"{directory_with_events}/154_failures_now_with_ICGS_events_classification.csv"
    main(directory_with_events_=directory_with_events, master_file_=master_file, output_csv_=output_csv)
    print("Done")



import VBMicrolensing
import pandas as pd
import numpy as np
import math
from scipy.optimize import minimize,least_squares
import RTModel
import shutil
import os
import matplotlib.pyplot as plt
import time
import logging
import sys

def minimize_linear_pars(y,err,x):
    """
    Exact solution to minimize linear eq chi2
    """
    alpha1 = np.sum(x/err**2)
    alpha2 = np.sum(1/err**2)
    alpha3 = np.sum(y/err**2)
    alpha4 = np.sum((x/err)**2)
    alpha5 = np.sum(x*y/err**2)
    a = (alpha5*alpha2 - alpha1*alpha3)/(alpha4*alpha2-alpha1**2)
    b = (alpha4*alpha3 - alpha5*alpha1)/(alpha4*alpha2-alpha1**2)
    return a,b

def get_chi2(magnitude_list,mag_data_list,error_list,ndatasets):
    """calculate chi2 for model on each dataset provided"""
    chi2_list = []
    for i in range(ndatasets):
        res = (magnitude_list[i] - mag_data_list[i]) / error_list[i]
        chi2 = np.sum(np.power(res,2))
        chi2_list.append(chi2)
    chi2_sum = np.sum(chi2_list)
    return chi2_list,chi2_sum

def get_residuals(magnitude_list,mag_data_list,error_list,ndatasets):
    """calculate chi2 for model on each dataset provided"""
    residue_list = []
    for i in range(ndatasets):
        res = (magnitude_list[i] - mag_data_list[i]) / error_list[i]
        residue_list.append(res)
    residues = np.concatenate(residue_list)
    return residues

def calculate_magnifications(pars,a1_list,data_list,ndatasets,VBMInstance):
    """Calculate LC magnitudes in each passband provided for 2L1S"""
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    for i in range(ndatasets):
        VBMInstance.a1 = a1_list[i]
        data = data_list[i]
        magnifications, y1, y2 = VBMInstance.BinaryLightCurve(pars,data[:,-1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        #Fix so minimize_linear_parameters uses flux_err not mag_err
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    return magnitude_list,source_flux_list,blend_flux_list

def calculate_magnifications_pspl(pars,data_list,data_mag,ndatasets,VBMInstance):
    """Calculate LC magnitudes in each passband provided for PSPL model"""
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    pars_log = [np.log(pars[0]), np.log(pars[1]), pars[2]]
    for i in range(ndatasets):
        data = data_list[i]
        magnifications, y1, y2 = VBMInstance.PSPLLightCurve(pars_log,data[:,-1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        magnifications, y1, y2 = VBMInstance.PSPLLightCurve(pars_log,data_mag)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    return magnitude_list,source_flux_list,blend_flux_list

def evaluate_model(psplpars, fsblpars, a1_list, data_list, VBMInstance, pspl_chi2):
    """ Calculate chi2 for one model in the grid
        fsblpars = [s,q,alpha,tstar]
        psplpars = follows VBMicrolensing format
        data_list list with a dataframe for each band in RTModel data format
        ndatasets = number of datasets
    """
    #blend_flux_list = []
    #source_flux_list = []
    ndatasets = len(data_list)
    pars = [np.log(fsblpars[0]), np.log(fsblpars[1]), psplpars[0], fsblpars[2], np.log(fsblpars[3] / psplpars[1]), math.log(psplpars[1]), psplpars[2]]
    magnitude_list,source_flux_list,blend_flux_list = calculate_magnifications(pars,a1_list,data_list,ndatasets,VBMInstance)
    magdata_list = []
    magerr_list = []
    for i in range(ndatasets):
        magdata_list.append(data_list[i][:,0])
        magerr_list.append(data_list[i][:,1])
    chi2_list,chi2_sum = get_chi2(magnitude_list, magdata_list, magerr_list, ndatasets)
    return_list = np.concatenate((pars, source_flux_list,blend_flux_list, chi2_list,[chi2_sum, chi2_sum-pspl_chi2]))
    return return_list

def grid_fit(event_path, dataset_list, pspl_pars, grid_s, grid_q, grid_alpha, tstar, a1_list, pspl_chi2):
    """
        Fit grid of models to determine initial conditions
    """
    data_list = []
    for i in range(len(dataset_list)):
        data_list.append(np.loadtxt(f'{event_path}/Data/{dataset_list[i]}'))
    VBMInstance = VBMicrolensing.VBMicrolensing()
    VBMInstance.RelTol = 1e-03
    VBMInstance.Tol=1e-03
    s,q,alpha = np.meshgrid(grid_s,grid_q,grid_alpha)
    s= s.flatten()
    q = q.flatten()
    alpha = alpha.flatten()
    #print(s.shape[0])
    grid_results = np.zeros(shape=(s.shape[0],9+3*len(a1_list)))
    print(f'PSPL pars: {pspl_pars[0]} {pspl_pars[1]} {pspl_pars[2]}')
    print(f'Checking {s.shape[0]} Models on grid!')
    for i in range(s.shape[0]):
        fsblpars = [s[i],q[i],alpha[i],tstar]
        print(f'{i} {s[i]} {q[i]} {alpha[i]} {tstar}')
        output = evaluate_model(pspl_pars, fsblpars, a1_list, data_list, VBMInstance, pspl_chi2)
        grid_results[i,:] = output
        #if i%5000==0: print(f'{i} Models checked')
    print('Done checking models!')
    return grid_results

def pspl_fit(event_path,dataset_list,p0=None,init_ind = 0,method='lm'):
    """ Find best fitting PSPL Model"""
    data_list = []
    for i in range(len(dataset_list)):
        data_list.append(np.loadtxt(f'{event_path}/Data/{dataset_list[i]}'))
    VBMInstance = VBMicrolensing.VBMicrolensing()
    VBMInstance.RelTol = 1e-03
    VBMInstance.Tol=1e-03
    if p0 is None:
        print('Setting default initial conditions')
        t0_init = data_list[init_ind][np.argmin(data_list[init_ind][:,0]),-1]
        p0 = [0.1,10,t0_init]
    else:
        print('Using provided initial conditions')
    #magnitude_list = []
    #source_flux_list = []
    #blend_flux_list = []
    if method =='lm':
        print('Using Levenberg-Marquardt')
        res = least_squares(fun=calc_pspl_residuals, x0=p0, args=[[data_list,len(data_list),VBMInstance]],method='lm',ftol=1e-10, xtol=1e-10, gtol=1e-10, max_nfev=50000)
    else:
        print('Using some scipy.minimize')
        res = minimize(calc_pspl_chi2, p0,[data_list,len(data_list),VBMInstance],method=method)
    print('Done Fitting PSPL!')
    return res

def calc_pspl_chi2(pars,args):
    """ Objective function to optimize for pspl_fit() """
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    data_list = args[0]
    ndatasets = args[1]
    VBMInstance = args[2]
    #logger = args[3]
    pars_log = [np.log(pars[0]),np.log(pars[1]),pars[2]]
    #logger.info(pars)
    for i in range(ndatasets):
        data = data_list[i]
        magnifications,y1,y2 = VBMInstance.PSPLLightCurve(pars_log,data[:,-1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    magdata_list = []
    magerr_list = []
    for i in range(ndatasets):
        magdata_list.append(data_list[i][:,0])
        magerr_list.append(data_list[i][:,1])
    chi2_list, chi2 = get_chi2(magnitude_list, magdata_list, magerr_list, ndatasets)
    print(chi2)
    return chi2

def calc_pspl_residuals(pars,args):
    """ Objective function to optimize for pspl_fit() """
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    data_list = args[0]
    ndatasets = args[1]
    VBMInstance = args[2]
    #logger = args[3]
    pars_log = [np.log(pars[0]),np.log(pars[1]),pars[2]]
    #print(pars)
    for i in range(ndatasets):
        data = data_list[i]
        magnifications,y1,y2 = VBMInstance.PSPLLightCurve(pars_log,data[:,-1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    magdata_list = []
    magerr_list = []
    for i in range(ndatasets):
        magdata_list.append(data_list[i][:,0])
        magerr_list.append(data_list[i][:,1])
    residuals = get_residuals(magnitude_list, magdata_list, magerr_list, ndatasets)
    chi2 = np.sum(np.power(residuals,2))
    print(f'chi2: {chi2}')
    return residuals


def filter_by_q(grid_result,pspl_thresh=-50):
    """
    Finds best initial conditions on grid for each value of q.
    Checks s>1 s<1 even though RTModel will do its own offset degeneracy check.
    """
    print('Picking best models for each mass ratio')
    q_list = list(grid_result['log(q)'].unique())
    best_grid_models = []
    # check for best s>1 s<1 solution for each q
    for q in q_list:
        subgrid = grid_result[grid_result['log(q)'] == q]
        subgrid_sge1 = subgrid[grid_result['log(s)'] >=1]# filter results for 1 q value.
        best_model = subgrid_sge1.sort_values('delta_pspl_chi2').reset_index(drop=True).iloc[0,
                     :]  # get best grid model for this q
        if best_model['delta_pspl_chi2'] <= pspl_thresh:
            best_grid_models.append(best_model)
        subgrid_sl1 = subgrid[grid_result['log(s)'] < 1]  # filter results for 1 q value.
        best_model = subgrid_sl1.sort_values('delta_pspl_chi2').reset_index(drop=True).iloc[0,
                     :]  # get best grid model for this q
        if best_model['delta_pspl_chi2'] <= pspl_thresh:
            best_grid_models.append(best_model)
    best_grid_models = pd.concat(best_grid_models, axis=1,
                                 ignore_index=True).T  # create a DataFrame of the best grid models
    # convert to initial condition input format used by RTModel
    init_cond = best_grid_models.iloc[:, 0:7].values.copy()
    init_cond[:, 0] = np.exp(init_cond[:, 0])
    init_cond[:, 1] = np.exp(init_cond[:, 1])
    init_cond[:, 4] = np.exp(init_cond[:, 4])
    init_cond[:, 5] = np.exp(init_cond[:, 5])
    print('Done')
    return best_grid_models, init_cond

def run_event(event_path,dataset_list,grid_s,grid_q,grid_alpha,tstar,a1_list,pspl_thresh,processors,method='lm',archive=True):
    """ Wrapper Function to go from pspl_fit to final RTModel runs."""
    # First do the PSPL fit
    pspl_results = pspl_fit(event_path=event_path,dataset_list=dataset_list,method=method)
    if method == 'lm':
        pspl_chi2 = pspl_results.cost*2
    else: pspl_chi2 = pspl_results.chi2
    pspl_pars = pspl_results.x
    rtm = RTModel.RTModel()
    rtm.set_processors(nprocessors=processors)
    rtm.set_event(event_path)
    if archive:
        rtm.archive_run()
        shutil.rmtree(f'{event_path}/run-0001')

    #save pspl fit to a txt file
    time0 = time.time()
    grid_fit_results = grid_fit(event_path=event_path, dataset_list=dataset_list, pspl_pars=pspl_pars,
                                grid_s=grid_s,grid_q=grid_q,grid_alpha=grid_alpha,tstar=tstar,a1_list=a1_list,pspl_chi2=pspl_chi2)
    time1 = time.time()
    print(f'ICGS time: {time1-time0}')
    time0 = time.time()
    names =  ['log(s)','log(q)','u0','alpha','log(rho)','log(tE)','t0','fs0','fb0','fs1','fb1','fs2','fb2','chi20','chi21','chi22','chi2sum','delta_pspl_chi2']
    grid_result_df = pd.DataFrame(grid_fit_results,columns=names)
    filtered_df,init_conds = filter_by_q(grid_result=grid_result_df,pspl_thresh=pspl_thresh)
    #Now run these in RTModel
    # Have RTModel prints go to log not stdout
 # have to remove old stuff or it affects the InitConds for everything.
    #Write some outputs after clearing the directory
    with open(f'{event_path}/Data/pspl_pars.txt','w') as f:
        f.write(f'{pspl_pars[0]},{pspl_pars[1]},{pspl_pars[2]},{pspl_chi2}')
    np.savetxt(fname=f'{event_path}/Data/ICGS_initconds.txt', X=init_conds)  # save init conds to a text file
    np.savetxt(f'{event_path}/Data/grid_fit.txt', grid_fit_results)
    modeltypes = ['PS','BS','BO','LS','LX','LO']
    peak_threshold = 5
    rtm.set_processors(nprocessors=processors)
    rtm.set_event(event_path)
    rtm.config_Reader(otherseasons=0, binning=1000000,renormalize=0)
    rtm.config_InitCond(peakthreshold=peak_threshold,modelcategories=modeltypes)
    rtm.Reader()
    rtm.InitCond()
    #Do FSPL fit for comparison
    print('Launching PS Fits')
    rtm.launch_fits('PS')
    rtm.ModelSelector('PS')
    print('Launching BS Fits')
    rtm.launch_fits('BS')
    rtm.ModelSelector('BS')
    print('Launching BO Fits')
    rtm.launch_fits('BO')
    rtm.ModelSelector('BO')
    print('Launching LS Fits')
    num_init_cond = init_conds.shape[0]
    for n in range(num_init_cond):
        init_cond = list(init_conds[n,:])
        #launch each fit from the init conds
        rtm.LevMar(f'LSfit{n:03}',parameters = init_cond)
    rtm.ModelSelector('LS')
    print('Launching LX and LO fits')
    rtm.launch_fits('LX')
    rtm.ModelSelector('LX')
    rtm.launch_fits('LO')
    rtm.ModelSelector('LO')
    rtm.Finalizer()
    print('Done')
    time1 = time.time()
    print(f'RTModel time: {time1 - time0}')
    return None


def plot_pspl(pars,dataset,event_path):
    data = np.loadtxt(f'{event_path}/Data/{dataset}')
    VBMInstance = VBMicrolensing.VBMicrolensing()
    VBMInstance.RelTol = 1e-03
    VBMInstance.Tol = 1e-03
    tmag = np.linspace(data[0,-1],data[-1,-1],80000)
    magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_pspl(pars,[data],tmag,1,VBMInstance)
    magnitude = magnitude_list[0]
    fig, ax = plt.subplots(dpi=100, layout='tight')
    ax.plot(tmag, magnitude, zorder=10, label='PSPL Fit', color='black')
    #magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_pspl([0.6291636,50.20121621,pars[-1]], [data],tmag, 1, VBMInstance)
    #magnitude = magnitude_list[0]
    #ax.plot(tmag, magnitude, zorder=10, label='PSPL True', color='red')
    ax.errorbar(data[:, -1], y=data[:, 0], yerr=data[:, 1], marker='.', markersize=0.75,
                linestyle=' ', label='W146')
    ax.set_xlim(pars[-1]-1*pars[1],pars[-1]+1*pars[1])
    ax.yaxis.set_inverted(True)
    ax.legend()
    plt.savefig(f'{event_path}/pspl_plot.png')
    plt.show()
    return None




def calculate_magnifications_plots(pars,a1_list,data_list,data_mag,ndatasets,VBMInstance,parallax=False):
    """Calculate LC magnitudes in each passband provided for 2L1S"""
    magnitude_list = []
    source_flux_list = []
    blend_flux_list = []
    for i in range(ndatasets):
        VBMInstance.a1 = a1_list[i]
        data = data_list[i]
        if parallax == False:
            magnifications, y1, y2 = VBMInstance.BinaryLightCurve(pars,data[:,-1])
        else:
            magnifications, y1, y2, sorb = VBMInstance.BinaryLightCurveOrbital(pars, data[:, -1])
        meas_flux = 10**(-0.4*data[:,0])
        meas_flux_err = 0.4*np.log(10)*10**(-0.4*data[:,0])*data[:,1]
        #Fix so minimize_linear_parameters uses flux_err not mag_err
        source_flux,blend_flux = minimize_linear_pars(meas_flux,meas_flux_err,magnifications)
        magnifications, y1, y2 = VBMInstance.BinaryLightCurve(pars, data_mag)
        sim_flux = np.array(magnifications)*source_flux+blend_flux
        source_flux_list.append(source_flux)
        blend_flux_list.append(blend_flux)
        sim_magnitudes = -2.5*np.log10(sim_flux)
        magnitude_list.append(sim_magnitudes)
    return magnitude_list,source_flux_list,blend_flux_list

def plot_2L1S(pars,dataset,event_path,xrange,yrange,evname):
    data = np.loadtxt(f'{event_path}/Data/{dataset}')
    VBMInstance = VBMicrolensing.VBMicrolensing()
    VBMInstance.RelTol = 1e-04
    VBMInstance.Tol = 1e-05
    tmag = np.linspace(pars[-1]-5*pars[5],pars[-1]+5*pars[5],80000)
    a1 = 0.33
    magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_plots(pars,[a1],[data],tmag,1,VBMInstance)
    magnitude = magnitude_list[0]
    fig, ax = plt.subplots(dpi=100, layout='tight')
    ax.plot(tmag, magnitude, zorder=10, label='2L1S Fit', color='black')
    #magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_pspl([0.6291636,50.20121621,pars[-1]], [data],tmag, 1, VBMInstance)
    #magnitude = magnitude_list[0]
    #ax.plot(tmag, magnitude, zorder=10, label='PSPL True', color='red')
    ax.errorbar(data[:, -1], y=data[:, 0], yerr=data[:, 1], marker='.', markersize=0.75,
                linestyle=' ', label='W146')
    if yrange == None:
        ind_0 = np.where(tmag >= xrange[0])
        ind_1 = np.where(tmag <= xrange[1])

        ind_data_0 = np.where(data[:,-1] >= xrange[0])
        ind_data_1 = np.where(data[:, -1] <= xrange[1])
        ind_plot = np.intersect1d(ind_0,ind_1)
        ind_data = np.intersect1d(ind_data_0,ind_data_1)

        ymin = np.min([np.min(magnitude[ind_plot]) - 0.1,np.min(data[ind_data,0]) - 0.1])
        ymax = np.max([np.max(magnitude[ind_plot]) + 0.1,np.max(data[ind_data,0]) + 0.1])
        yrange=[ymin,ymax]
    ax.set_xlim(xrange[0],xrange[1])
    ax.set_ylim(yrange[0], yrange[1])
    ax.yaxis.set_inverted(True)
    ax.set_ylabel('W146')
    ax.set_xlabel("HJD'")
    ax.legend()
    plt.savefig(f'{event_path}/{evname}')
    plt.show()
    return None

def plot_2L1S_parallax(pars,dataset,event_path,xrange,yrange,evname):
    data = np.loadtxt(f'{event_path}/Data/{dataset}')
    VBMInstance = VBMicrolensing.VBMicrolensing()
    VBMInstance.RelTol = 1e-04
    VBMInstance.Tol = 1e-05
    coordinatefile = f'{event_path}/Data/event.coordinates'
    VBMInstance.SetObjectCoordinates(coordinatefile)
    tmin = pars[6]-5*pars[5]
    tmax = pars[6]+5*pars[5]
    if tmin < data[0,-1]:
        tmin = data[0,-1]
    if tmax > data[-1,-1]:
        tmax = data[-1,-1]
    tmag = np.linspace(tmin,tmax,80000)
    a1 = 0.33
    magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_plots(pars,[a1],[data],tmag,1,VBMInstance,parallax=True)
    magnitude = magnitude_list[0]
    fig, ax = plt.subplots(dpi=100, layout='tight')
    ax.plot(tmag, magnitude, zorder=10, label='2L1S Fit', color='black')
    #magnitude_list, source_flux_list, blend_flux_list = calculate_magnifications_pspl([0.6291636,50.20121621,pars[-1]], [data],tmag, 1, VBMInstance)
    #magnitude = magnitude_list[0]
    #ax.plot(tmag, magnitude, zorder=10, label='PSPL True', color='red')
    ax.errorbar(data[:, -1], y=data[:, 0], yerr=data[:, 1], marker='.', markersize=0.75,
                linestyle=' ', label='W146')
    # Set some good ylim based on the xlim if yrange==None.
    if yrange == None:
        ind_0 = np.where(tmag >= xrange[0])
        ind_1 = np.where(tmag <= xrange[1])

        ind_data_0 = np.where(data[:,-1] >= xrange[0])
        ind_data_1 = np.where(data[:, -1] <= xrange[1])
        ind_plot = np.intersect1d(ind_0,ind_1)
        ind_data = np.intersect1d(ind_data_0,ind_data_1)

        ymin = np.min([np.min(magnitude[ind_plot]) - 0.1,np.min(data[ind_data,0]) - 0.1])
        ymax = np.max([np.max(magnitude[ind_plot]) + 0.1,np.max(data[ind_data,0]) + 0.1])
        yrange=[ymin,ymax]
    ax.set_xlim(xrange[0],xrange[1])
    ax.set_ylim(yrange[0], yrange[1])
    ax.yaxis.set_inverted(True)
    ax.set_ylabel('W146')
    ax.set_xlabel("HJD'")
    ax.legend()
    plt.savefig(f'{event_path}/{evname}')
    plt.show()
    return None


from dataclasses import dataclass
import numpy as np

from moana.lens import ResonantCaustic as CausticCalculator
from moana.lens import close_limit_2l, wide_limit_2l


@dataclass
class Caustic:
    """
    This class is a wrapper for the caustic_calculator function from the moana package. It takes in
    the separation_s and mass_ratio_q and calculate the upper and lower half of the caustic,
    or as an alternative the horizontal and vertical components of the caustic.
    """
    type_of_caustics: str
    separation_s: float
    mass_ratio_q: float
    number_of_data_points: int
    half_caustic: np.ndarray
    upper_half: tuple
    lower_half: tuple
    horizontal: np.ndarray
    vertical: np.ndarray

    def __init__(self, separation_s, mass_ratio_q, number_of_data_points=10000):
        self.type_of_caustics = None
        self.separation_s = separation_s
        self.mass_ratio_q = mass_ratio_q
        caustic_data_points = CausticCalculator(sep=separation_s, q=mass_ratio_q)
        self.number_of_data_points = number_of_data_points
        caustic_data_points._sample(self.number_of_data_points)
        self.half_caustic = caustic_data_points.full['zeta'].values
        self.upper_half = np.real(self.half_caustic), np.imag(self.half_caustic)
        self.lower_half = np.real(self.half_caustic), -np.imag(self.half_caustic)

        self.horizontal = np.concatenate((np.real(self.half_caustic), np.real(self.half_caustic)))
        self.vertical = np.concatenate((np.imag(self.half_caustic), -np.imag(self.half_caustic)))

    def define_topology(self):
        """
        Check the topology type of the binary lens system based on moana
        :return:
        """
        if self.separation_s < close_limit_2l(self.mass_ratio_q):
            self.type_of_caustics = 'close'
        elif self.separation_s <= wide_limit_2l(self.mass_ratio_q):
            self.type_of_caustics = 'resonant'
        elif self.separation_s > wide_limit_2l(self.mass_ratio_q):
            self.type_of_caustics = 'wide'
        else:
            self.type_of_caustics = 'Unknown'
        return self.type_of_caustics


if __name__ == '__main__':
    caustic_object = Caustic(separation_s=0.8, mass_ratio_q=0.3, number_of_data_points=100)
    import matplotlib.pyplot as plt

    # plt.scatter(caustic_object.upper_half[0], caustic_object.upper_half[1], s=0.1)
    # plt.scatter(caustic_object.lower_half[0], caustic_object.lower_half[1], s=0.1)
    plt.scatter(caustic_object.horizontal, caustic_object.vertical)
    plt.show()
    type_of_caustics = caustic_object.define_topology()
    print(type_of_caustics)



"""
This wrap up the RTModel runs
Note you have to run classes_and_files_reader/RTModel_results_reader.py first.
"""
import os
import pandas as pd

def collect_chi2_data(base_path, output_file):
    """
    Collect chi2 data from the best LX and LO from each event folder and save to a summary CSV file.
    :param base_path:
    :param output_file:
    :return:
    """
    # Dictionary to store results
    results = []

    # Loop through event_* folders
    for event_folder in sorted(os.listdir(base_path)):
        event_path = os.path.join(base_path, event_folder)
        models_path = os.path.join(event_path, "Models", "chi2_top1_of_each_binary_lens_model.csv")

        # Check if the expected file exists
        if os.path.isdir(event_path) and os.path.exists(models_path):
            df = pd.read_csv(models_path)

            # Extract best LX model and chi2
            best_lx_row = df[df['model'].str.startswith('LX')]
            if not best_lx_row.empty:
                best_lx, best_lx_chi2 = best_lx_row.iloc[0]['model'], best_lx_row.iloc[0]['chi2']
            else:
                best_lx, best_lx_chi2 = None, None  # Handle missing LX case

            # Extract best LO model and chi2
            best_lo_row = df[df['model'].str.startswith('LO')]
            if not best_lo_row.empty:
                best_lo, best_lo_chi2 = best_lo_row.iloc[0]['model'], best_lo_row.iloc[0]['chi2']
            else:
                best_lo, best_lo_chi2 = None, None  # Handle missing LO case

            results.append([event_folder, best_lx, best_lx_chi2, best_lo, best_lo_chi2])

        # Create DataFrame and save
        output_df = pd.DataFrame(results, columns=["event_name", "best_LX", "best_LX_chi2", "best_LO", "best_LO_chi2"])
        output_df.to_csv(output_file, index=False)


def finding_best_rtmodel_fit_name(*, event_name_='event_0_90_1748',
                                  model_type_= 'LX',
                                  best_rtmodel_fit_table_path_='/Users/stela/Documents/Scripts/orbital_task/RTModel_runs/sample_rtmodel_v2.4_LX_LO_summary.csv'):
    """
    Find the best fit model name from the table of best models. Basically reads the table above
    :param best_rtmodel_fit_table_path_:
    :param model_type_:
    :param event_name_:
    :return:
    """
    # Read the table
    best_rtmodel_fit_table = pd.read_csv(best_rtmodel_fit_table_path_)
    # Filter the table for the given event name and model type
    event_row = best_rtmodel_fit_table[(best_rtmodel_fit_table['event_name'] == event_name_)]
    # Get the best model name
    best_rtmodel_fit_name = event_row[f'best_{model_type_}'].values[0]
    return best_rtmodel_fit_name

if __name__ == '__main__':
    # Example usage

    ### TO SAVE
    # computer_path = "/discover/nobackup/sishitan/orbital_task"
    computer_path = "/"
    run_type = 'sample_rtmodel_v2.4'

    base_path = f"{computer_path}/RTModel_runs/{run_type}"
    output_file = f"{computer_path}/RTModel_runs/{run_type}_LX_LO_summary.csv"
    collect_chi2_data(base_path, output_file)
    print(f"Summary file saved at {output_file}")

    ### TO READ
    event_name = 'event_0_90_1748'
    best_rtmodel_fit_table_path = f'{computer_path}/RTModel_runs/{run_type}_LX_LO_summary.csv'
    model_name = finding_best_rtmodel_fit_name(event_name_=event_name,
                                    model_type_='LX',
                                    best_rtmodel_fit_table_path_=best_rtmodel_fit_table_path)
    print(f"Best LX model for {event_name}: {model_name}")


import os
import csv

def main(base_dir,  output_file):
    results = []
    for root, dirs, files in os.walk(base_dir):
        if "pspl_pars.txt" in files:
            file_path = os.path.join(root, "pspl_pars.txt")
            event_name = os.path.basename(os.path.dirname(root))  # e.g., event_0_917_1851

            try:
                with open(file_path, "r") as f:
                    line = f.readline().strip()
                    if line:
                        parts = line.split(",")
                        last_value = parts[-1]
                        results.append((event_name, last_value))
            except Exception as e:
                print(f"Error reading {file_path}: {e}")

    # Sort by event name (optional)
    results.sort(key=lambda x: x[0])

    # Save results to CSV
    with open(output_file, "w", newline="") as csvfile:
        writer = csv.writer(csvfile)
        writer.writerow(["event_name", "pspl_chi2_passed"])
        writer.writerows(results)

    print(f"✅ Saved summary to {output_file} with {len(results)} entries.")

if __name__ == "__main__":
    # computer_path = '/gpfsm/dnb34/sishitan/orbital_task'
    computer_path = '/Users/stela/Documents/Scripts/orbital_task'
    base_dir = f"{computer_path}/RTModel_runs/154_failures_v24_v31/second_run"
    output_file = os.path.join(base_dir, "pspl_chi2_summary.csv")
    main(base_dir, output_file)


import sys
from gridfit import run_event
import numpy as np
import pandas as pd
import warnings
warnings.filterwarnings("ignore", message="Polyfit may be poorly conditioned")

def main(event_name_, events_directory_path_, satellite_files_directory_path_, alpha_grid_density_):
    """Run a grid search for a given event."""
    a1_list = [0.33, 0.4534, 0.2517]
    dataset_list = ['RomanW146sat1.dat', 'RomanZ087sat2.dat', 'RomanK213sat3.dat']
    s = '0.37 0.40 0.43 0.46 0.49 0.52 0.56 0.60 0.65 0.70 0.75 0.80 0.84 0.88 0.91 0.93 0.95 0.965 0.98 0.99 1.00 1.01 1.02 1.035 1.05 1.07 1.09 1.12 1.15 1.18 1.22 1.26 1.30 1.34 1.38 1.42 1.46 1.50 1.55 1.60 1.66 1.72 1.79 1.86 1.94 2.02 2.1 2.2 2.3 2.4 2.5 2.62 2.75'
    s = s.split(' ')
    s_grid = np.array(list(map(float, s)))
    eps1 = '1.e-6 3.16e-6 1.e-5 3.16e-5 1.e-4 3.16e-4 1.e-3 3.16e-3 1.e-2 3.16e-2'
    eps1 = eps1.split(' ')
    eps1_grid = np.array(list(map(float, eps1)))
    q_grid = 1 / ((1 / eps1_grid) - 1)
    tstar = 0.03
    alpha_grid = np.linspace(-3.15, 3.15, alpha_grid_density_)
    event_path = f'{events_directory_path_}/{event_name_}'
    run_event(event_path=event_path, dataset_list=dataset_list, grid_s=s_grid, grid_q=q_grid, grid_alpha=alpha_grid,
              tstar=tstar,
              a1_list=a1_list, pspl_thresh=0, satellitedir=satellite_files_directory_path_, processors=8,
              parallax=False)


if __name__ == "__main__":
    # dir6 = '/Users/jmbrashe/Downloads/events'
    # event = 'event_0_87_1723_memtest'
    run_index = int(sys.argv[1])
    alpha_grid_density = int(sys.argv[2])

    events_directory_path = f'/gpfsm/dnb34/sishitan/orbital_task/RTModel_runs/269_problematic_events_ICGS_{alpha_grid_density}'
    satellite_files_directory_path = '/gpfsm/dnb34/sishitan/orbital_task/data/satellitedir'

    table_path = f'{events_directory_path}/problematic_events.txt'
    list_of_events_ = pd.read_table(table_path, header=None)
    event_name = list_of_events_.iloc[run_index, 0]

    # with multiprocessing.Pool(1) as p:
    #    p.map(main,events)
    main(event_name, events_directory_path, satellite_files_directory_path, alpha_grid_density)



import pandas as pd


def df_filter(df_, property_and_intervals):
    # Filter rows based on the intervals for tha property
    # e.g. ['Lens_Mass', 0.2, 0.5]
    filtered_df = df_[(df_[property_and_intervals[0]] >= property_and_intervals[1]) & (
            df_[property_and_intervals[0]] <= property_and_intervals[2])]
    return filtered_df


def mass_and_distance_filter(general_path_, file_path_, is_saving=True):
    df = pd.read_csv(general_path_ + file_path_)
    lens_mass = ['Lens_Mass', 0.2, 0.5]
    df_mass = df_filter(df, lens_mass)
    lens_distance = ['Lens_Dist', 1.0, 5.0]
    df_distance = df_filter(df_mass, lens_distance)
    if is_saving:
        df_distance.to_csv(general_path_ + 'most_common_by_lens_mass_distance.csv', index=False)
    else:
        return df_distance


def make_sbatch_commands(project_path_, general_path_, file_path_):
    filtered_df = mass_and_distance_filter(project_path_ + general_path_, file_path_, is_saving=False)
    bash_path = f'{project_path_}bash_scripts'

    # Open the file for writing
    with open(f'{bash_path}/submission_common.sh', 'w') as f:
        # Iterate over the rows of the DataFrame
        for index, row in filtered_df.iterrows():
            # Extract the values for SubRun, Field, and EventID
            subrun = row['SubRun']
            field = row['Field']
            event_id = row['EventID']

            # Write the sbatch command to the file
            f.write(f'sbatch bash_scripts/rtmodel_new.sh {subrun} {field} {event_id}\n')

    print(f"Submission script saved to {bash_path}/submission_common.sh")


def make_sample_sbatch_commands(project_path_, general_path_, file_path_):
    sample_df = pd.read_csv(project_path_ + general_path_ + file_path_)
    filtered_df = mass_and_distance_filter(project_path_ + general_path_, file_path_, is_saving=False)

    # Merge the two dataframes to find common rows
    common_rows = sample_df.merge(filtered_df)
    # Drop the common rows from representative_sample_df
    sample_df_cleaned = sample_df.loc[~sample_df.index.isin(common_rows.index)]

    bash_path = f'{project_path_}bash_scripts'

    # Open the file for writing
    with open(f'{bash_path}/submission_sample.sh', 'w') as f:
        # Iterate over the rows of the DataFrame
        for index, row in sample_df_cleaned.iterrows():
            # Extract the values for SubRun, Field, and EventID
            subrun = row['SubRun']
            field = row['Field']
            event_id = row['EventID']

            # Write the sbatch command to the file
            f.write(f'sbatch bash_scripts/rtmodel_new.sh {subrun} {field} {event_id}\n')

    print(f"Submission script saved to {bash_path}/submission_sample.sh")


if __name__ == "__main__":
    project_path = '//'
    general_path = 'data/gulls_orbital_motion_extracted/'
    file_path = 'OMPLDG_croin_cassan.sample.csv'
    # mass_and_distance_filter(project_path+general_path_outputs, file_path)
    # make_sbatch_commands(project_path, general_path_outputs, file_path)
    make_sample_sbatch_commands(project_path, general_path, file_path)



import pandas as pd
import re


def master_head_getter(path_to_wfirstcolumn_file_):
    """
    This function reads the wfirstcolumn file and returns a pandas dataframe with the columns
    :param path_to_wfirstcolumn_file_:
    :return:
    """
    wfirstcolumn_df = pd.read_csv(path_to_wfirstcolumn_file_, sep="\s+")
    return wfirstcolumn_df


def master_file_spliter(path_to_master_file_, path_to_wfirstcolumn_file_, path_to_output_folder_):
    """
    This function reads the master file and splits it into 4 different files based on the type of event.
    :param path_to_master_file_:
    :param path_to_wfirstcolumn_file_:
    :param path_to_output_folder_:
    :return:
    """
    single_lens_lines = []  # dcnormffp
    binary_star_lines = []  # ombin
    bound_planet_lines = []  # omcassan
    cataclysmic_variables_lines = []  # dccv
    header_line = str(master_head_getter(path_to_wfirstcolumn_file_)['name'].values)
    header_line = header_line.replace(' \'|\' ', ',').replace('\' \'', ',').replace('\'', '')
    header_line = header_line.replace(' ', '').replace('|', '').replace('\n', ',')
    header_line = header_line.replace('[','').replace(']','').replace(',,', ',')
    print()
    with open(path_to_master_file_, "r") as master_file:
        for line in master_file:
            line = line.replace(' | ', ' ')
            if "Sky position" in line:
                line = line.replace(' ', ' ')
                line = re.sub("\s+", ",", line.strip())
                master_header_line = line
            else:
                line = line.replace(' ', ',')
                if "dcnormffp" in line:
                    single_lens_lines.append(line)
                elif "ombin" in line:
                    binary_star_lines.append(line)
                elif "omcassan" in line:
                    bound_planet_lines.append(line)
                elif "dccv" in line:
                    cataclysmic_variables_lines.append(line)

                else:
                    raise ValueError("Unknown type of event\n", line)

    with open(f'{path_to_output_folder_}/single_lens.csv', 'w') as single_lens_file:
        single_lens_header_line = header_line+(',unimportant0,unimportant1,unimportant2,'
                                               'event_type,unimportant3,lc_root,data_challenge_lc_number\n')
        single_lens_file.write(single_lens_header_line)
        for line in single_lens_lines:
            single_lens_file.write(line)

    with open(f'{path_to_output_folder_}/binary_star.csv', 'w') as binary_star_file:
        binary_lens_header_line = header_line.replace(',normw,sigma_t0,sigma_tE,sigma_u0,sigma_alpha,sigma_s,'
                                                      'sigma_q,sigma_rs,sigma_F00,sigma_fs0,sigma_F01,sigma_fs1,'
                                                      'sigma_thetaE',
                                                      ',normw,event_type,unimportant0,lc_root,'
                                                      'data_challenge_lc_number\n')
        binary_star_file.write(binary_lens_header_line)
        for line in binary_star_lines:
            line = line.replace(',,,', ',')
            binary_star_file.write(line)

    with open(f'{path_to_output_folder_}/bound_planet.csv', 'w') as bound_planet_file:
        bound_planet_header_line = header_line.replace(',normw,sigma_t0,sigma_tE,sigma_u0,sigma_alpha,sigma_s,'
                                                       'sigma_q,sigma_rs,sigma_F00,sigma_fs0,sigma_F01,sigma_fs1,'
                                                       'sigma_thetaE',
                                                       ',normw,event_type,unimportant0,lc_root,'
                                                       'data_challenge_lc_number\n')
        bound_planet_file.write(bound_planet_header_line)
        for line in bound_planet_lines:
            line = line.replace(',,,', ',')
            bound_planet_file.write(line)

    with open(f'{path_to_output_folder_}/cataclysmic_variables.csv', 'w') as cataclysmic_variables_file:
        cataclysmic_variables_header_line = header_line.replace(',normw,sigma_t0,sigma_tE,sigma_u0,sigma_alpha,sigma_s,'
                                                                'sigma_q,sigma_rs,sigma_F00,sigma_fs0,sigma_F01,'
                                                                'sigma_fs1,'
                                                                'sigma_thetaE',
                                                                ',event_type,unimportant0,lc_root,'
                                                                'data_challenge_lc_number\n')
        cataclysmic_variables_file.write(cataclysmic_variables_header_line)
        for line in cataclysmic_variables_lines:
            cataclysmic_variables_file.write(line)


if __name__ == '__main__':
    # Write where you can find the master files
    root_path = '/Users/stela/Documents/Scripts/RTModel_project/datachallenge'
    path_to_master_file = f'{root_path}/master_file.txt'
    path_to_wfirstcolumn_file = f'{root_path}/wfirstColumnNumbers.txt'
    path_to_output_folder = root_path
    master_file_spliter(path_to_master_file, path_to_wfirstcolumn_file, path_to_output_folder)
    print("Files have been created successfully")





#from gridfit import *
#import sys
#import tdqm
import numpy as np
from jasmine.initial_condition_grid_search.gridfit_moa_ogle import *


def main(event_path):
    moar_ald = 0.7259
    ogleI_ald = 0.6155
    ogleV_ald = 0.6730
    a1_list = [moar_ald,ogleI_ald,ogleV_ald]
    dataset_list = ['moar.dat','ogleI.dat','ogleV.dat']
    s = '0.37 0.40 0.43 0.46 0.49 0.52 0.56 0.60 0.65 0.70 0.75 0.80 0.84 0.88 0.91 0.93 0.95 0.965 0.98 0.99 1.00 1.01 1.02 1.035 1.05 1.07 1.09 1.12 1.15 1.18 1.22 1.26 1.30 1.34 1.38 1.42 1.46 1.50 1.55 1.60 1.66 1.72 1.79 1.86 1.94 2.02 2.1 2.2 2.3 2.4 2.5 2.62 2.75'
    s = s.split(' ')
    s_grid = list(map(float,s))
    eps1 = '1.e-6 3.16e-6 1.e-5 3.16e-5 1.e-4 3.16e-4 1.e-3 3.16e-3 1.e-2 3.16e-2 1.e-1 3.16e-1'
    eps1 = eps1.split(' ')
    eps1_grid = np.array(list(map(float,eps1)))
    q_grid = 1/((1/eps1_grid)-1)
    tstar=0.03
    alpha_grid = np.linspace(-3.15,3.15,316)
    dir6 = '/Users/jmbrashe/VBBOrbital/NEWGULLS/6_events_for_testing'
    event_name=event_path
    event_path = f'{dir6}/{event_path}'
    run_event(event_path=event_path, dataset_list=dataset_list, grid_s=s_grid, grid_q=q_grid, grid_alpha=alpha_grid, tstar=tstar,
              a1_list=a1_list, pspl_thresh=0,processors=1)
if __name__ == "__main__":
    #dir6 = '/Users/jmbrashe/VBBOrbital/NEWGULLS/6_events_for_testing/levmar_runs_redo'
    event = 'event_0_5_606'
    #with multiprocessing.Pool(1) as p:
    #    p.map(main,events)
    main(event)









import pandas as pd
import numpy as np


def bound_planet_master_reader(filepath_='./OMPLLD_croin_cassan.sample.csv'):
    """
    This function reads the bound planet file and returns a pandas dataframe with the columns
    :param filepath_:
    :return: bound_planet_df
    """
    bound_planet_df = pd.read_csv(filepath_)
    return bound_planet_df


def lightcurve_data_reader(SubRun, Field, ID, *, folder_path_='data', include_ephem=True):
    """
    This function reads the lightcurve data file and returns a pandas dataframe with the BJD Magnitude Error columns
    :param folder_path_:
    :param data_challenge_lc_number_:
    :param filter_:
    :return: lightcurve_data_df
    """
    fname = f'OMPLDG_croin_cassan_{SubRun}_{Field}_{ID}.det.lc'

    with open(f'{folder_path_}/{fname}', 'r') as f:
        fs = f.readline()
        fs = np.array(fs.split(' ')[1:4]).astype(float)
        f.readline()
        f.readline()
        m_source = f.readline()
        m_source = np.array(m_source.split(' ')[1:4]).astype(float)
    if include_ephem:
        columns = ['Simulation_time', 'measured_relative_flux', 'measured_relative_flux_error', 'true_relative_flux',
                   'true_relative_flux_error', 'observatory_code', 'saturation_flag', 'best_single_lens_fit',
                   'parallax_shift_t', 'parallax_shift_u', 'BJD', 'source_x', 'source_y', 'lens1_x', 'lens1_y',
                   'lens2_x', 'lens2_y', 'SAT_X', 'SAT_Y', 'SAT_Z']
    else:
        columns = ['Simulation_time', 'measured_relative_flux', 'measured_relative_flux_error', 'true_relative_flux',
                   'true_relative_flux_error', 'observatory_code', 'saturation_flag', 'best_single_lens_fit',
                   'parallax_shift_t', 'parallax_shift_u', 'BJD', 'source_x', 'source_y', 'lens1_x', 'lens1_y',
                   'lens2_x', 'lens2_y']
    lightcurve_data_df = pd.read_csv(f'{folder_path_}/{fname}', names=columns, comment='#', sep='\s+')
    lightcurve_data_df['days'] = lightcurve_data_df['BJD'] - 2450000
    lightcurve_list = []
    for observatory_code in range(3):
        obs_data = lightcurve_data_df[lightcurve_data_df['observatory_code'] == observatory_code]
        mag_constant = m_source[observatory_code] + 2.5 * np.log10(fs[observatory_code])
        if np.min(obs_data['measured_relative_flux']) < 0:
            obs_data.loc[:, 'measured_relative_flux'] = obs_data['measured_relative_flux'] - np.min(
                obs_data['measured_relative_flux']) + 1e-5
        measured_relative_flux_error = obs_data['measured_relative_flux_error']
        mag = - 2.5 * np.log10(obs_data['measured_relative_flux']) + mag_constant
        mag_err = (2.5 / (np.log(10))) * measured_relative_flux_error / obs_data['measured_relative_flux']
        obs_data.insert(loc=obs_data.shape[1], column='mag', value=mag)
        obs_data.insert(loc=obs_data.shape[1], column='mag_err', value=mag_err)
        lightcurve_list.append(obs_data[['mag', 'mag_err', 'days']])
    return lightcurve_list


def microlensing_signal_reader(SubRun, Field, ID, *, folder_path_='data', include_ephem=True):
    fname = f'OMPLDG_croin_cassan_{SubRun}_{Field}_{ID}.det.lc'
    if include_ephem:
        columns = ['Simulation_time', 'measured_relative_flux', 'measured_relative_flux_error', 'true_relative_flux',
                   'true_relative_flux_error', 'observatory_code', 'saturation_flag', 'best_single_lens_fit',
                   'parallax_shift_t', 'parallax_shift_u', 'BJD', 'source_x', 'source_y', 'lens1_x', 'lens1_y',
                   'lens2_x',
                   'lens2_y']
    else:
        columns = ['Simulation_time', 'measured_relative_flux', 'measured_relative_flux_error', 'true_relative_flux',
                   'true_relative_flux_error', 'observatory_code', 'saturation_flag', 'best_single_lens_fit',
                   'parallax_shift_t', 'parallax_shift_u', 'BJD', 'source_x', 'source_y', 'lens1_x', 'lens1_y',
                   'lens2_x', 'lens2_y']
    lightcurve_data_df = pd.read_csv(f'{folder_path_}/{fname}', names=columns, comment='#', sep='\s+')
    lightcurve_list = []
    for observatory_code in range(3):
        obs_data = lightcurve_data_df[lightcurve_data_df['observatory_code'] == observatory_code]
        lightcurve_list.append(obs_data[['Simulation_time', 'true_relative_flux', 'true_relative_flux_error']])
    return lightcurve_list


def whole_columns_lightcurve_reader(SubRun, Field, ID, *, folder_path_='data', include_ephem=True):
    """
    This function reads the lightcurve data file and returns a pandas dataframe with the BJD Magnitude Error columns
    :param folder_path_:
    :param data_challenge_lc_number_:
    :param filter_:
    :return: lightcurve_data_df
    """
    fname = f'OMPLDG_croin_cassan_{SubRun}_{Field}_{ID}.det.lc'

    with open(f'{folder_path_}/{fname}', 'r') as f:
        fs = f.readline()
        fs = np.array(fs.split(' ')[1:4]).astype(float)
        f.readline()
        f.readline()
        m_source = f.readline()
        m_source = np.array(m_source.split(' ')[1:4]).astype(float)
    if include_ephem:
        columns = ['Simulation_time', 'measured_relative_flux', 'measured_relative_flux_error', 'true_relative_flux',
                   'true_relative_flux_error', 'observatory_code', 'saturation_flag', 'best_single_lens_fit',
                   'parallax_shift_t', 'parallax_shift_u', 'BJD', 'source_x', 'source_y', 'lens1_x', 'lens1_y',
                   'lens2_x', 'lens2_y', 'SAT_X', 'SAT_Y', 'SAT_Z']
    else:
        columns = ['Simulation_time', 'measured_relative_flux', 'measured_relative_flux_error', 'true_relative_flux',
                   'true_relative_flux_error', 'observatory_code', 'saturation_flag', 'best_single_lens_fit',
                   'parallax_shift_t', 'parallax_shift_u', 'BJD', 'source_x', 'source_y', 'lens1_x', 'lens1_y',
                   'lens2_x', 'lens2_y']
    lightcurve_data_df = pd.read_csv(f'{folder_path_}/{fname}', names=columns, comment='#', sep='\s+')
    lightcurve_data_df['days'] = lightcurve_data_df['BJD'] - 2450000
    return lightcurve_data_df


if __name__ == '__main__':
    print('')
    # lc_type = what_type_of_lightcurve(2)
    # print(lc_type)



from bokeh.models import HoverTool, ColumnDataSource, Whisker, BoxZoomTool, ResetTool, PanTool, SaveTool, WheelZoomTool
from bokeh.plotting import figure
from bokeh.palettes import Category10

import numpy as np

from jasmine.classes_and_files_reader.new_gullsrges_reader import whole_columns_lightcurve_reader


def plotting_mass_ratio(q_and_s_summary_df, model_type, x_range=None, y_range=None):
    if model_type == 'LS':
        title = f'Mass-ratio - 2L1S ({model_type})'
    elif model_type == 'LX':
        title = f'Mass-ratio - 2L1S+parallax ({model_type})'
    elif model_type == 'LO':
        title = f'Mass-ratio - 2L1S+parallax+OM ({model_type})'
    else:
        raise ValueError(f'model_type {model_type} should be LS, LX or LO')

    upper = 10 ** (np.log10(q_and_s_summary_df[f'{model_type}_q']) + q_and_s_summary_df[f'{model_type}_q_err'])
    lower = 10 ** (np.log10(q_and_s_summary_df[f'{model_type}_q']) - q_and_s_summary_df[f'{model_type}_q_err'])
    # uper = q_and_s_summary_df[f'{model_type}_q'] + q_and_s_summary_df[f'{model_type}_q_err']
    # lower = q_and_s_summary_df[f'{model_type}_q'] - q_and_s_summary_df[f'{model_type}_q_err']
    source = ColumnDataSource(data=dict(
        x=q_and_s_summary_df['true_q'],
        y=q_and_s_summary_df[f'{model_type}_q'],
        upper=upper,
        lower=lower,
        desc=q_and_s_summary_df.index,
    ))
    hover = HoverTool(tooltips=[
        ("(true,model)", "($x, $y)"),
        ("LC#", "@desc")
    ])
    if x_range is None:
        p = figure(width=350, height=350, title=title, x_axis_label='true q', y_axis_label=f'{model_type} q',
                   x_axis_type='log', y_axis_type='log',
                   tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool(), WheelZoomTool()])
    else:
        p = figure(width=350, height=350, title=title, x_axis_label='true q', y_axis_label=f'{model_type} q',
                   x_axis_type='log', y_axis_type='log',
                   tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool(), WheelZoomTool()], x_range=x_range, y_range=y_range)
    largest_true_q = q_and_s_summary_df['true_q'].max()
    smallest_true_q = q_and_s_summary_df['true_q'].min()
    p.line([smallest_true_q, largest_true_q + 1e-6], [smallest_true_q, largest_true_q + 1e-6], line_width=1, line_color='red')
    p.scatter('x', 'y', size=5, fill_alpha=0.6, source=source)
    whisker_errorbar = Whisker(source=source, base="x", upper="upper", lower="lower",
                               line_width=1.0, line_alpha=1.0)  # level="overlay",
    p.add_layout(whisker_errorbar)
    return p


def plotting_separation(q_and_s_summary_df, model_type, x_range=None, y_range=None):
    if model_type == 'LS':
        title = f'Separation - 2L1S ({model_type})'
    elif model_type == 'LX':
        title = f'Separation - 2L1S+parallax ({model_type})'
    elif model_type == 'LO':
        title = f'Separation - 2L1S+parallax+OM ({model_type})'
    else:
        raise ValueError(f'model_type {model_type} should be LS, LX or LO')
    source = ColumnDataSource(data=dict(
        x=q_and_s_summary_df['true_s'],
        y=q_and_s_summary_df[f'{model_type}_s'],
        upper=q_and_s_summary_df[f'{model_type}_s'] + q_and_s_summary_df[f'{model_type}_s_err'],
        lower=q_and_s_summary_df[f'{model_type}_s'] - q_and_s_summary_df[f'{model_type}_s_err'],
        desc=q_and_s_summary_df.index,
    ))
    hover = HoverTool(tooltips=[
        ("(true,model)", "($x, $y)"),
        ("LC#", "@desc")
    ])
    if x_range is None:
        p = figure(width=350, height=350, title=title, x_axis_label='true s', y_axis_label=f'{model_type} s',
                   tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool()])
    else:
        p = figure(width=350, height=350, title=title, x_axis_label='true s', y_axis_label=f'{model_type} s',
                   tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool()], x_range=x_range, y_range=y_range)
        # add a circle renderer with vectorized colors and sizes
    largest_true_s = q_and_s_summary_df['true_s'].max()
    smallest_true_s = q_and_s_summary_df['true_s'].min()
    p.line([smallest_true_s, largest_true_s + 1e-6], [smallest_true_s, largest_true_s + 1e-6], line_width=1, line_color='red')
    p.scatter('x', 'y', size=5, fill_alpha=0.6, source=source)
    whisker_errorbar = Whisker(source=source, base="x", upper="upper", lower="lower",
                               line_width=1.0, line_alpha=1.0)  # level="overlay",
    p.add_layout(whisker_errorbar)
    return p


def plotting_pien(parallax_summary_df, model_type, x_range=None, y_range=None):
    if model_type == 'LX':
        title = f'piEN - 2L1S+parallax ({model_type})'
    elif model_type == 'LO':
        title = f'piEN - 2L1S+parallax+OM ({model_type})'
    else:
        raise ValueError(f'model_type {model_type} should be LS, LX or LO')
    source = ColumnDataSource(data=dict(
        x=parallax_summary_df['true_piEN'],
        y=parallax_summary_df[f'{model_type}_piEN'],
        upper=parallax_summary_df[f'{model_type}_piEN'] + parallax_summary_df[f'{model_type}_piEN_error'],
        lower=parallax_summary_df[f'{model_type}_piEN'] - parallax_summary_df[f'{model_type}_piEN_error'],
        desc=parallax_summary_df.index,
    ))
    hover = HoverTool(tooltips=[
        ("(true,model)", "($x, $y)"),
        ("LC#", "@desc")
    ])
    if x_range is None:
        p = figure(width=350, height=350, title=title, x_axis_label='true pien', y_axis_label=f'{model_type} pien',
                   tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool()])
    else:
        p = figure(width=350, height=350, title=title, x_axis_label='true pien', y_axis_label=f'{model_type} pien',
                   tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool()], x_range=x_range, y_range=y_range)
    p = figure(width=350, height=350, title=title, x_axis_label='true pien', y_axis_label=f'{model_type} pien',
               tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool()])
    largest_true_piE = parallax_summary_df['true_piEN'].max()
    smallest_true_piE = parallax_summary_df['true_piEN'].min()
    p.line([smallest_true_piE, largest_true_piE], [smallest_true_piE, largest_true_piE],
           line_width=1, line_color='red')
    p.scatter('x', 'y', size=5, fill_alpha=0.6, source=source)
    whisker_errorbar = Whisker(source=source, base="x", upper="upper", lower="lower",
                               line_width=1.0, line_alpha=1.0)  # level="overlay",
    p.add_layout(whisker_errorbar)
    return p


def plotting_piee(parallax_summary_df, model_type, x_range=None, y_range=None):
    if model_type == 'LX':
        title = f'piEE - 2L1S+parallax ({model_type})'
    elif model_type == 'LO':
        title = f'piEE - 2L1S+parallax+OM ({model_type})'
    else:
        raise ValueError(f'model_type {model_type} should be LS, LX or LO')
    source = ColumnDataSource(data=dict(
        x=parallax_summary_df['true_piEE'],
        y=parallax_summary_df[f'{model_type}_piEE'],
        upper=parallax_summary_df[f'{model_type}_piEE'] + parallax_summary_df[f'{model_type}_piEE_error'],
        lower=parallax_summary_df[f'{model_type}_piEE'] - parallax_summary_df[f'{model_type}_piEE_error'],
        desc=parallax_summary_df.index,
    ))
    hover = HoverTool(tooltips=[
        ("(true,model)", "($x, $y)"),
        ("LC#", "@desc")
    ])
    if x_range is None:
        p = figure(width=350, height=350, title=title, x_axis_label='true piee', y_axis_label=f'{model_type} piee',
                   tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool()])
    else:
        p = figure(width=350, height=350, title=title, x_axis_label='true piee', y_axis_label=f'{model_type} piee',
                   tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool()], x_range=x_range, y_range=y_range)
    p = figure(width=350, height=350, title=title, x_axis_label='true piee', y_axis_label=f'{model_type} piee',
               tools=[hover, BoxZoomTool(), ResetTool(), PanTool(), SaveTool()])
    largest_true_piE = parallax_summary_df['true_piEN'].max()
    smallest_true_piE = parallax_summary_df['true_piEN'].min()
    p.line([smallest_true_piE, largest_true_piE], [smallest_true_piE, largest_true_piE],
           line_width=1, line_color='red')
    p.scatter('x', 'y', size=5, fill_alpha=0.6, source=source)
    whisker_errorbar = Whisker(source=source, base="x", upper="upper", lower="lower",
                               line_width=1.0, line_alpha=1.0)  # level="overlay",
    p.add_layout(whisker_errorbar)
    return p


def plotting_measured_and_true_relative_flux(subrun, field, event_id, sample_folder):
    lightcurve = whole_columns_lightcurve_reader(SubRun=subrun, Field=field, ID=event_id,
                                                 folder_path_=sample_folder,
                                                 include_ephem=True)

    # Assuming 'lightcurve' is your DataFrame
    # Create a custom mapping for observatory_code
    observatory_map = {0: 'W146', 1: 'Z087', 2: 'K213'}

    # Generate colors for each observatory_code using Category10 palette
    color_map = {0: Category10[3][0], 1: Category10[3][1], 2: Category10[3][2]}

    # Add observatory names and colors to the dataframe
    lightcurve['observatory_name'] = lightcurve['observatory_code'].map(observatory_map)
    lightcurve['color'] = lightcurve['observatory_code'].map(color_map)

    # Darken the color for the error bars
    lightcurve['dark_color'] = lightcurve['color'].apply(darken_color)

    # Create ColumnDataSource for measured flux
    source_measured = ColumnDataSource(data={
        'days': lightcurve['days'],
        'measured_relative_flux': lightcurve['measured_relative_flux'],
        'measured_relative_flux_error': lightcurve['measured_relative_flux_error'],
        'color': lightcurve['color'],
        'dark_color': lightcurve['dark_color'],
        'observatory_name': lightcurve['observatory_name']
    })

    # Create ColumnDataSource for true flux
    source_true = ColumnDataSource(data={
        'days': lightcurve['days'],
        'true_relative_flux': lightcurve['true_relative_flux'],
        'true_relative_flux_error': lightcurve['true_relative_flux_error'],
        'color': lightcurve['color'],
        'dark_color': lightcurve['dark_color'],
        'observatory_name': lightcurve['observatory_name']
    })

    # Create the first plot (measured_relative_flux)
    p1 = figure(
        title="Measured Flux with Error Bars",
        x_axis_label="Days",
        y_axis_label="Measured Relative Flux",
        height=400,
        width=800
    )

    # Plot measured flux with color based on observatory_code
    p1.scatter('days', 'measured_relative_flux', size=3, color='color', alpha=0.8, legend_field='observatory_name',
               source=source_measured)

    # Add error bars using Whiskers (following your example for upper and lower bounds)
    upper_measured = [x + e for x, e in
                      zip(lightcurve['measured_relative_flux'], lightcurve['measured_relative_flux_error'])]
    lower_measured = [x - e for x, e in
                      zip(lightcurve['measured_relative_flux'], lightcurve['measured_relative_flux_error'])]

    source_measured.add(upper_measured, 'upper')
    source_measured.add(lower_measured, 'lower')

    # Create the whiskers with darkened colors matching the data points
    whisker_measured = Whisker(source=source_measured, base="days", upper="upper", lower="lower",
                               line_width=0.5, line_color='dark_color',
                               line_alpha=0.3)  # Darker error bars with transparency
    whisker_measured.upper_head.line_color = 'dark_color'
    whisker_measured.lower_head.line_color = 'dark_color'
    p1.add_layout(whisker_measured)

    p1.grid.grid_line_alpha = 0.3
    p1.legend.location = "top_left"

    # Create the second plot (true_relative_flux)
    p2 = figure(
        title="True Flux with Error Bars",
        x_axis_label="Days",
        y_axis_label="True Relative Flux",
        height=400,
        width=800,
        x_range=p1.x_range  # Ensuring both plots share the same x-axis range
    )

    # Plot true flux with color based on observatory_code
    p2.scatter('days', 'true_relative_flux', size=3, color='color', alpha=0.8, legend_field='observatory_name',
               source=source_true)

    # Add error bars using Whiskers (following your example for upper and lower bounds)
    upper_true = [x + e for x, e in zip(lightcurve['true_relative_flux'], lightcurve['true_relative_flux_error'])]
    lower_true = [x - e for x, e in zip(lightcurve['true_relative_flux'], lightcurve['true_relative_flux_error'])]

    source_true.add(upper_true, 'upper')
    source_true.add(lower_true, 'lower')

    # Create the whiskers with darkened colors matching the data points
    whisker_true = Whisker(source=source_true, base="days", upper="upper", lower="lower",
                           line_width=0.5, line_color='dark_color',
                           line_alpha=0.3)  # Darker error bars with transparency
    whisker_true.upper_head.line_color = 'dark_color'
    whisker_true.lower_head.line_color = 'dark_color'
    p2.add_layout(whisker_true)

    p2.grid.grid_line_alpha = 0.3
    p2.legend.location = "top_left"
    return p1, p2


# Function to darken a color by adjusting its RGB components
def darken_color(hex_color, factor=0.8):
    # Convert hex to RGB
    rgb = [int(hex_color[i:i + 2], 16) for i in (1, 3, 5)]
    # Darken by multiplying RGB values
    rgb_darker = [max(int(x * factor), 0) for x in rgb]
    # Convert back to hex
    return '#{:02x}{:02x}{:02x}'.format(*rgb_darker)